/*
 *  Copyright (C) 2006-2007 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
*/

/**
 * KibaDrawableObject:
 *
 * @KibaDrawableObject is the base class for the objects on @KibaDock.
 */

#ifndef __KIBA_DRAWABLE_OBJECT_H__
#define __KIBA_DRAWABLE_OBJECT_H__

#include <gtk/gtk.h>
#include <glib.h>
#include <cairo.h>

#include <kiba-object.h>
#include <kiba-cairo-utils.h>

G_BEGIN_DECLS

typedef struct _KibaDrawableObject KibaDrawableObject;
typedef struct _KibaDrawableObjectClass KibaDrawableObjectClass;

/**
 * KibaDrawableObjectPaintFunc:
 * @Param1: The KibaDrawableObject Instance.
 * @Param2: The drawing context.
 * @Param3: User Data.
 *
 * Method used for custom rendering.
 */
typedef void (*KibaDrawableObjectPaintFunc) (KibaDrawableObject*, cairo_t*, gpointer);

/**
 * KibaDrawableObjectCustomRotateFunc:
 * @Param1: The KibaDrawableObject Instance.
 * @Param2: The rotate angle return.
 * @Param3: User Data.
 *
 * Method used for custom rotation.
 */
typedef void (*KibaDrawableObjectCustomRotateFunc) (KibaDrawableObject*, float*, gpointer);

#define KIBA_TYPE_DRAWABLE_OBJECT           (kiba_drawable_object_get_type ())
#define KIBA_DRAWABLE_OBJECT(obj)           (G_TYPE_CHECK_INSTANCE_CAST ((obj), KIBA_TYPE_DRAWABLE_OBJECT, KibaDrawableObject))
#define KIBA_DRAWABLE_OBJECT_CLASS(obj)     (G_TYPE_CHECK_CLASS_CAST ((obj), KIBA_TYPE_DRAWABLE_OBJECT,  KibaDrawableObjectClass))
#define KIBA_IS_DRAWABLE_OBJECT(obj)        (G_TYPE_CHECK_INSTANCE_TYPE ((obj), KIBA_TYPE_DRAWABLE_OBJECT))
#define KIBA_IS_DRAWABLE_OBJECT_CLASS(obj)  (G_TYPE_CHECK_CLASS_TYPE ((obj), KIBA_TYPE_DRAWABLE_OBJECT))
#define KIBA_DRAWABLE_OBJECT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), KIBA_TYPE_DRAWABLE_OBJECT, KibaDrawableObjectClass))

/**
 * KibaDrawableObjectCustomPaintData:
 *
 * Type for custom rendering.
 */
typedef struct {
	KibaDrawableObjectPaintFunc func;
	gpointer data;
}KibaDrawableObjectCustomPaintData;

/**
 * KibaDrawableObjectCustomRotateData:
 *
 * Type for custom rotate animations.
 */
typedef struct {
	KibaDrawableObjectCustomRotateFunc func;
	gpointer data;
}KibaDrawableObjectCustomRotateData;

struct _KibaDrawableObject {
	KibaObject parent;
	
	float alpha;            /**< current alpha value */
	float scroll_alpha;     /**< Alpha for the scroll fade */
	KibaRGBA shadow_color;
	gint shadow_affection;
	gint shadow_offset;
	
	CairoSurfaceData *surface_data;   /**< The object surface */
	cairo_surface_t *switch_surface; /**< used for the switch animation */
	
	GList *custom_rotate_funcs; /**< list of all KibaDrawableObjectCustomRotateData's */
	
	KibaDrawableObjectPaintFunc paint;                     /**< the paint function */
	gpointer paint_data;                /**< the paint data */
	GList *custom_paint_funcs_after;  /**< custom paint methods before the paint method */
	GList *custom_paint_funcs_before; /**< custom paint methods after the paint method */
};

struct _KibaDrawableObjectClass {
	KibaObjectClass parent_class;
	
	/* signals */
	void (* switch_event) (KibaDrawableObject *object);
};

GtkWidget* kiba_drawable_object_new (const char *group);

void kiba_drawable_object_set_alpha (KibaDrawableObject *object,
                                     float alpha);

void kiba_drawable_object_queue_draw (KibaDrawableObject *object);

void kiba_drawable_object_paint (KibaDrawableObject *object,
                                 cairo_t *cr,
                                 gpointer data);

void kiba_drawable_object_paint_reflection (KibaDrawableObject *object,
                                            cairo_t *cr);

void kiba_drawable_object_paint_gear (KibaDrawableObject *object,
                                      cairo_t *cr);

void kiba_drawable_object_paint_shadow (KibaDrawableObject *object,
                                        cairo_t *cr);

void kiba_drawable_object_paint_with_prelight (KibaDrawableObject *object,
                                               cairo_surface_t *surface,
                                               cairo_t *cr);

void kiba_drawable_object_rotate (KibaDrawableObject *object,
                                  cairo_t *cr);

gboolean kiba_drawable_object_scale (KibaDrawableObject *object,
                                     cairo_t *cr);

void kiba_drawable_object_switch_surface (KibaDrawableObject *object);

void kiba_drawable_object_set_paint_function (KibaDrawableObject *object,
                                              KibaDrawableObjectPaintFunc  func,
                                              gpointer data);

void kiba_drawable_object_add_custom_rotate_function (KibaDrawableObject *object,
                                                      KibaDrawableObjectCustomRotateFunc func,
                                                      gpointer user_data);

void kiba_drawable_object_remove_custom_rotate_function_by_data (KibaDrawableObject *object,
                                                                 gpointer user_data);

void kiba_drawable_object_add_custom_paint_function (KibaDrawableObject *object,
                                                     KibaDrawableObjectPaintFunc func,
                                                     gboolean before,
                                                     gpointer user_data);

void kiba_drawable_object_remove_custom_paint_function_by_data (KibaDrawableObject *object,
                                                                gpointer user_data);

G_END_DECLS

#endif
